<?php

namespace FiloBlu\Cegid\Helper\Cegid;

use FiloBlu\Cegid\Helper\Cegid\Base as BaseHelper;
use FiloBlu\CegidY2\Service\CredentialProvider;
use FiloBlu\CegidY2\Service\ServiceProvider;
use FiloBlu\CegidY2\Api\CustomerOrderManagementService\ServiceType\Create;
use FiloBlu\CegidY2\Api\CustomerOrderManagementService\StructType\RetailContextFactory;
use FiloBlu\CegidY2\Api\CustomerOrderManagementService\StructType\CreateFromFactory;
use FiloBlu\CegidY2\Api\CustomerOrderManagementService\StructType\RetailContext;
use FiloBlu\CegidY2\Api\CustomerOrderManagementService\StructType\CreateFromRequest;
use FiloBlu\CegidY2\Api\CustomerOrderManagementService\StructType\CreateFromRequestHeader;
use FiloBlu\CegidY2\Api\CustomerOrderManagementService\StructType\CreateFromRequestHeaderOriginalDocumentIdentifier;
use FiloBlu\CegidY2\Api\CustomerOrderManagementService\StructType\CreateFromRequestHeaderOriginalDocumentIdentifierReference;
use FiloBlu\CegidY2\Api\CustomerOrderManagementService\EnumType\CustomerOrderDocumentTypeOriginTransfo;
use FiloBlu\CegidY2\Api\CustomerOrderManagementService\EnumType\CustomerOrderDocumentTypeCreat;
use FiloBlu\CegidY2\Api\CustomerOrderManagementService\StructType\CreateFromRequestLineCollection;
use FiloBlu\CegidY2\Api\CustomerOrderManagementService\StructType\CreateFromRequestLine;
use FiloBlu\CegidY2\Api\CustomerOrderManagementService\StructType\CreateFromRequestLineItemIdentifier;
use FiloBlu\Cegid\Helper\Cegid\Order as OrderHelper;
use Magento\Sales\Api\Data\OrderInterface;

/**
 * Class OrderShipping
 * @package FiloBlu\Cegid\Helper\Cegid
 */
class OrderShipping extends BaseHelper
{
    /**
     * @var CredentialProvider
     */
    protected $credentialProvider;
    /**
     * @var ServiceProvider
     */
    protected $createService;
    /**
     * @var RetailContextFactory
     */
    protected $retailContextFactory;
    /**
     * @var OrderHelper
     */
    protected $orderHelper;
    /**
     * @var CreateFromFactory
     */
    protected $createFromFactory;

    // Init variables
    protected $order_source_items_quantities;
    protected $is_groupage_order;
    protected $is_main_source_order;
    protected $is_source_order;
    protected $cegid_customer_id;
    protected $stock_id;
    protected $current_order;
    // Temp variables
    protected $current_source_code;
    protected $current_array_quantities;

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        CredentialProvider $credentialProvider,
        RetailContextFactory $retailContextFactory,
        ServiceProvider $createService,
        OrderHelper $orderHelper,
        CreateFromFactory $createFromFactory
    ){
        parent::__construct($context);
        $this->credentialProvider = $credentialProvider;
        $this->retailContextFactory = $retailContextFactory;
        $this->createService = $createService; //Service provider
        $this->orderHelper = $orderHelper;
        $this->createFromFactory = $createFromFactory;
    }

    public function initShipment(OrderInterface $order){
        $this->order_source_items_quantities = $this->orderHelper->getOrderSourceItemsQuantities($order);
        $this->is_groupage_order = $this->orderHelper->isGroupageOrder($order);
        $this->is_main_source_order = $this->orderHelper->isMainSourceOrder($order);
        $this->is_source_order = $this->orderHelper->isSourceOrder($order);
        $this->cegid_customer_id = $this->orderHelper->getCustomerCegidFromOrder($order);
        $this->stock_id = $this->orderHelper->extInvHelperData->getStockIdByOrder($order);
        $this->current_order = $order;
    }

    public function createShipments(OrderInterface $order){
        $this->initShipment($order);
        $errors = array();
        foreach ($this->order_source_items_quantities as $source_code => $order_source_items_quantity) {
            $this->current_source_code = $source_code;
            $result = $this->createShipment($order_source_items_quantity);
            if ($result !== true){
                $errors[] = $result;
            }
        }

        if (!empty($errors)){
            return false;
        }
        return true;
    }

    public function createShipment($array_of_quantities_to_ship){

        $request = new CreateFromRequest();
        $request->setHeader($this->getHeader());
        $request->setLines($this->getLines($array_of_quantities_to_ship));

        $parameters = $this->createFromFactory->create();
        $parameters->setContext($this->getRetailContext());
        $parameters->setRequest($request);

        // Execute
        $createService = $this->createService->create();
        $createService->CreateFrom($parameters);
        // Save data
        $request = $createService->getLastRequest();
        $response = $createService->getLastResponse();
        $error = $createService->getLastError();
        $result = $createService->getResult();

        $this->writeLog(
            'createShipment invio di conferma spedizione a CEGID',
            $this->current_order,
            $request,
            $response
        );

        if(!$result && $error){
            return $error;
        }
        return true;
    }

    public function getHeader(){
        $createFromRequestHeader = new CreateFromRequestHeader();

        // TODO: date of? Order? Shipping?
        $date = new \DateTime($this->current_order->getCreatedAt());
        $createFromRequestHeader->setDate($date->format('Y-m-d'));

        $createFromRequestHeader->setExternalReference($this->orderHelper->getOrderReference($this->current_order,$this->current_source_code));
        $createFromRequestHeader->setFollowUpReference($this->orderHelper->getOrderReference($this->current_order,$this->current_source_code));
        $createFromRequestHeader->setInternalReference($this->orderHelper->getOrderReference($this->current_order,$this->current_source_code));
        $createFromRequestHeader->setWarehouseId("990");
        $createFromRequestHeader->setType(CustomerOrderDocumentTypeCreat::VALUE_CUSTOMER_DELIVERY);
        $createFromRequestHeader->setOriginalDocumentIdentifier($this->getOriginalDocumentIdentifier());

        return $createFromRequestHeader;
    }

    public function getOriginalDocumentIdentifier(){
        $reference = new CreateFromRequestHeaderOriginalDocumentIdentifierReference();
        $reference->setCustomerId($this->cegid_customer_id);
        $reference->setInternalReference($this->orderHelper->getOrderReference($this->current_order,$this->current_source_code));
        $reference->setType(CustomerOrderDocumentTypeOriginTransfo::VALUE_CUSTOMER_ORDER);

        $originalDocumentIdentifier = new CreateFromRequestHeaderOriginalDocumentIdentifier();
        $originalDocumentIdentifier->setReference($reference);
        return $originalDocumentIdentifier;
    }

    public function getLines($array_of_quantities_to_ship){

        $lines = new CreateFromRequestLineCollection();

        foreach ($array_of_quantities_to_ship as $sku => $data) {

            $barcode = $data['barcode'];
            $qty = $data['qty'];

            $itemIdentifier = new CreateFromRequestLineItemIdentifier();
            $itemIdentifier->setReference($barcode);

            $line = new CreateFromRequestLine();
            $line->setQuantity($qty);
            $line->setWarehouseId("990");
            $line->setItemIdentifier($itemIdentifier);

            $lines->addToLine($line);
        }

        return $lines;
    }

    /**
     * @return \FiloBlu\CegidY2\Api\CustomerOrderManagementService\StructType\RetailContext
     */
    public function getRetailContext()
    {
        return $this->retailContextFactory->create(['databaseId' =>
            $this->credentialProvider->getDatabase('customer_order_management_service')
        ]);
    }


}
